@@ -1,9 +1,7 @@
-# -*- coding: utf-8 -*-
 import atexit
 import os
 import pickle
 import shutil
-import sys
 import tempfile
 import textwrap
 import unittest
@@ -68,6 +66,8 @@ class TestGroup(unittest.TestCase):
         assert isinstance(g.info, InfoReporter)
         assert isinstance(repr(g.info), str)
         assert isinstance(g.info._repr_html_(), str)
+        if hasattr(store, 'close'):
+            store.close()
 
     def test_group_init_2(self):
         store, chunk_store = self.create_store()
@@ -79,12 +79,16 @@ class TestGroup(unittest.TestCase):
         assert '/foo/bar' == g.name
         assert 'bar' == g.basename
         assert isinstance(g.attrs, Attributes)
+        if hasattr(store, 'close'):
+            store.close()
 
     def test_group_init_errors_1(self):
         store, chunk_store = self.create_store()
         # group metadata not initialized
         with pytest.raises(ValueError):
             Group(store, chunk_store=chunk_store)
+        if hasattr(store, 'close'):
+            store.close()
 
     def test_group_init_errors_2(self):
         store, chunk_store = self.create_store()
@@ -92,6 +96,8 @@ class TestGroup(unittest.TestCase):
         # array blocks group
         with pytest.raises(ValueError):
             Group(store, chunk_store=chunk_store)
+        if hasattr(store, 'close'):
+            store.close()
 
     def test_create_group(self):
         g1 = self.create_group()
@@ -162,6 +168,9 @@ class TestGroup(unittest.TestCase):
         assert isinstance(g7, Group)
         assert g7.path == 'z'
 
+        if hasattr(g1.store, 'close'):
+            g1.store.close()
+
     def test_require_group(self):
         g1 = self.create_group()
 
@@ -203,6 +212,9 @@ class TestGroup(unittest.TestCase):
         assert isinstance(g7, Group)
         assert g7.path == 'z'
 
+        if hasattr(g1.store, 'close'):
+            g1.store.close()
+
     def test_create_dataset(self):
         g = self.create_group()
 
@@ -277,6 +289,9 @@ class TestGroup(unittest.TestCase):
         assert d.compressor.codec_id == 'zlib'
         assert 1 == d.compressor.level
 
+        if hasattr(g.store, 'close'):
+            g.store.close()
+
     def test_require_dataset(self):
         g = self.create_group()
 
@@ -321,6 +336,9 @@ class TestGroup(unittest.TestCase):
             g.require_dataset('foo', shape=1000, chunks=100, dtype='i2',
                               exact=True)
 
+        if hasattr(g.store, 'close'):
+            g.store.close()
+
     def test_create_errors(self):
         g = self.create_group()
 
@@ -374,6 +392,9 @@ class TestGroup(unittest.TestCase):
         with pytest.raises(PermissionError):
             g.require_dataset('zzz', shape=100, chunks=10)
 
+        if hasattr(g.store, 'close'):
+            g.store.close()
+
     def test_create_overwrite(self):
         try:
             for method_name in 'create_dataset', 'create', 'empty', 'zeros', \
@@ -397,6 +418,9 @@ class TestGroup(unittest.TestCase):
                                             overwrite=True)
                 assert (400,) == d.shape
                 assert isinstance(g['foo'], Group)
+
+                if hasattr(g.store, 'close'):
+                    g.store.close()
         except NotImplementedError:
             pass
 
@@ -624,6 +648,9 @@ class TestGroup(unittest.TestCase):
         assert g1.visitvalues(visitor1) is True
         assert g1.visititems(visitor1) is True
 
+        if hasattr(g1.store, 'close'):
+            g1.store.close()
+
     def test_empty_getitem_contains_iterators(self):
         # setup
         g = self.create_group()
@@ -634,6 +661,9 @@ class TestGroup(unittest.TestCase):
         assert 0 == len(g)
         assert 'foo' not in g
 
+        if hasattr(g.store, 'close'):
+            g.store.close()
+
     def test_iterators_recurse(self):
         # setup
         g1 = self.create_group()
@@ -658,6 +688,9 @@ class TestGroup(unittest.TestCase):
         assert 'zab' == arrays_recurse[0][0]
         assert g1['foo']['bar']['zab'] == arrays_recurse[0][1]
 
+        if hasattr(g1.store, 'close'):
+            g1.store.close()
+
     def test_getattr(self):
         # setup
         g1 = self.create_group()
@@ -670,6 +703,9 @@ class TestGroup(unittest.TestCase):
         # test that hasattr returns False instead of an exception (issue #88)
         assert not hasattr(g1, 'unexistingattribute')
 
+        if hasattr(g1.store, 'close'):
+            g1.store.close()
+
     def test_setitem(self):
         g = self.create_group()
         try:
@@ -685,6 +721,8 @@ class TestGroup(unittest.TestCase):
             assert 42 == g['foo'][()]
         except NotImplementedError:
             pass
+        if hasattr(g.store, 'close'):
+            g.store.close()
 
     def test_delitem(self):
         g = self.create_group()
@@ -703,6 +741,8 @@ class TestGroup(unittest.TestCase):
             assert 'foo' in g
             assert 'bar' not in g
             assert 'bar/baz' not in g
+        if hasattr(g.store, 'close'):
+            g.store.close()
 
     def test_move(self):
         g = self.create_group()
@@ -750,6 +790,9 @@ class TestGroup(unittest.TestCase):
         except NotImplementedError:
             pass
 
+        if hasattr(g.store, 'close'):
+            g.store.close()
+
     def test_array_creation(self):
         grp = self.create_group()
 
@@ -785,6 +828,9 @@ class TestGroup(unittest.TestCase):
         assert isinstance(j, Array)
         assert_array_equal(np.arange(100), j[:])
 
+        if hasattr(grp.store, 'close'):
+            grp.store.close()
+
         grp = self.create_group(read_only=True)
         with pytest.raises(PermissionError):
             grp.create('aa', shape=100, chunks=10)
@@ -809,6 +855,9 @@ class TestGroup(unittest.TestCase):
         with pytest.raises(PermissionError):
             grp.full_like('aa', a)
 
+        if hasattr(grp.store, 'close'):
+            grp.store.close()
+
     def test_paths(self):
         g1 = self.create_group()
         g2 = g1.create_group('foo/bar')
@@ -840,6 +889,9 @@ class TestGroup(unittest.TestCase):
         with pytest.raises(ValueError):
             g1['foo/../bar']
 
+        if hasattr(g1.store, 'close'):
+            g1.store.close()
+
     def test_pickle(self):
 
         # setup group
@@ -867,6 +919,9 @@ class TestGroup(unittest.TestCase):
         assert isinstance(g2['foo'], Group)
         assert isinstance(g2['foo/bar'], Array)
 
+        if hasattr(g2.store, 'close'):
+            g2.store.close()
+
     def test_context_manager(self):
 
         with self.create_group() as g:
@@ -934,8 +989,7 @@ class TestGroupWithZipStore(TestGroup):
 
         # Check that exiting the context manager closes the store,
         # and therefore the underlying ZipFile.
-        error = ValueError if sys.version_info >= (3, 6) else RuntimeError
-        with pytest.raises(error):
+        with pytest.raises(ValueError):
             store.zf.extractall()
 
 
@@ -1221,14 +1275,14 @@ def test_tree():
     g5.create_dataset('baz', shape=100, chunks=10)
 
     # test root group
-    expect_bytes = textwrap.dedent(u"""\
+    expect_bytes = textwrap.dedent("""\
     /
      +-- bar
      |   +-- baz
      |   +-- quux
      |       +-- baz (100,) float64
      +-- foo""").encode()
-    expect_text = textwrap.dedent(u"""\
+    expect_text = textwrap.dedent("""\
     /
      ├── bar
      │   ├── baz
@@ -1238,19 +1292,19 @@ def test_tree():
     _check_tree(g1, expect_bytes, expect_text)
 
     # test different group
-    expect_bytes = textwrap.dedent(u"""\
+    expect_bytes = textwrap.dedent("""\
     foo""").encode()
-    expect_text = textwrap.dedent(u"""\
+    expect_text = textwrap.dedent("""\
     foo""")
     _check_tree(g2, expect_bytes, expect_text)
 
     # test different group
-    expect_bytes = textwrap.dedent(u"""\
+    expect_bytes = textwrap.dedent("""\
     bar
      +-- baz
      +-- quux
          +-- baz (100,) float64""").encode()
-    expect_text = textwrap.dedent(u"""\
+    expect_text = textwrap.dedent("""\
     bar
      ├── baz
      └── quux
